Re: [PATCH 1/2] ref-filter: hacky "streaming" mode
- From
ZheNing Hu <adlternative@gmail.com>
- Date
- Sep 7, 2021, 05:42 UTC
- Message-ID
- <CAOLTT8QzNt8dv5-Q4nV9XU9Tq7KZVSjaYcM3T4F=F=xZ9A3UUA@mail.gmail.com>
- In-Reply-To
- <YTTC2IUO1ZmTOEoR@coredump.intra.peff.net>
Jeff King <peff@peff.net> 于2021年9月5日周日 下午9:15写道:
Show 70 quoted lines
>
> On Sun, Sep 05, 2021 at 04:20:02PM +0800, ZheNing Hu wrote:
>
> > > + if (ref_cbdata->filter->streaming_format) {
> > > + pretty_print_ref(refname, oid, ref_cbdata->filter->streaming_format);
> >
> > So we directly use pretty_print_ref() in streaming mode, OK.
> >
> > > + } else {
> > > + /*
> > > + * We do not open the object yet; sort may only need refname
> > > + * to do its job and the resulting list may yet to be pruned
> > > + * by maxcount logic.
> > > + */
> > > + ref = ref_array_push(ref_cbdata->array, refname, oid);
> > > + ref->commit = commit;
> > > + ref->flag = flag;
> > > + ref->kind = kind;
> > > + }
> > >
> > > return 0;
> > > }
> >
> > Therefore, in streaming mode, there is no need to push ref to
> > ref_array, which can
> > reduce the overhead of malloc(), free(), which makes sense.
>
> By the way, one thing I wondered here: how much of the benefit is from
> avoiding the ref_array, and how much is from skipping the sort entirely.
>
> It turns out that most of it is from the latter. If I do this:
>
> diff --git a/builtin/for-each-ref.c b/builtin/for-each-ref.c
> index 89cb6307d4..037d5db814 100644
> --- a/builtin/for-each-ref.c
> +++ b/builtin/for-each-ref.c
> @@ -78,7 +78,11 @@ int cmd_for_each_ref(int argc, const char **argv, const char *prefix)
> filter.name_patterns = argv;
> filter.match_as_path = 1;
> filter_refs(&array, &filter, FILTER_REFS_ALL | FILTER_REFS_INCLUDE_BROKEN);
> - ref_array_sort(sorting, &array);
> + /*
> + * we should skip this only when we are using the default refname
> + * sorting, but as an experimental hack, we'll just comment it out.
> + */
> + // ref_array_sort(sorting, &array);
>
> if (!maxcount || array.nr < maxcount)
> maxcount = array.nr;
>
> then the timings I get are:
>
> Benchmark #1: ./git.old for-each-ref --format='%(objectname) %(refname)'
> Time (mean ± σ): 341.4 ms ± 7.4 ms [User: 299.8 ms, System: 41.6 ms]
> Range (min … max): 333.5 ms … 355.1 ms 10 runs
>
> Benchmark #2: ./git.new for-each-ref --format='%(objectname) %(refname)'
> Time (mean ± σ): 249.1 ms ± 5.7 ms [User: 211.8 ms, System: 37.2 ms]
> Range (min … max): 245.9 ms … 267.0 ms 12 runs
>
> Summary
> './git.new for-each-ref --format='%(objectname) %(refname)'' ran
> 1.37 ± 0.04 times faster than './git.old for-each-ref --format='%(objectname) %(refname)''
>
> So of the 1.5x improvement that the original patch showed, 1.37x is from
> skipping the sort of the already-sorted data. I suspect that has less to
> do with sorting at all, and more to do with the fact that even just
> formatting "%(refname)" for each entry takes a non-trivial amount of
> time.
>Yes, I think this overhead may come from get_ref_atom_value() instead of QSORT_S().
> -Peff
Thanks. -- ZheNing